@@ -20,10 +20,10 @@
 #
 
 import unittest
-
+import doctest
 import sys
 import os
-from authheaders import authenticate_message, sign_message, get_domain_part
+from authheaders import authenticate_message, sign_message, get_domain_part, dmarc_lookup
 
 #import logging
 #logging.basicConfig(level=10)
@@ -50,6 +50,9 @@ class TestAuthenticateMessage(unittest.TestCase):
         self.message7 = read_test_data("test.message_np2")
         self.message8 = read_test_data("testcomma.message")
         self.message9 = read_test_data("test_nop.message")
+        self.message10 = read_test_data("test.bad.message")
+        self.message11 = read_test_data("test.nofrom.message")
+        self.message12 = read_test_data("test.gov.message")
         self.key = read_test_data("test.private")
 
     def dnsfunc(self, domain, timeout=5):
@@ -62,18 +65,22 @@ s8g3FgD2Ap3ZB5DekAo5wMmk4wimDO+U8QzI3SD07y2+07wlNWwIt8svnxgdxGkVbb\
 hzY8i+RQ9DpSVpPbF7ykQxtKXkv/ahW3KjViiAH+ghvvIhkx4xYSIc9oSwVmAl5Oct\
 MEeWUwg8Istjqz8BZeTWbf41fbNhte7Y+YqZOwq1Sd0DbvYAD9NOZK9vlfuac0598H\
 Y+vtSBczUiKERHv1yRbcaQtZFh5wtiRrN04BLUTD21MycBX5jYchHjPY/wIDAQAB""",
-          "_dmarc.example.com": """v=DMARC1\; p=reject\;\
- rua=mailto:dmarc.reports@valimail.com,mailto:dmarc_agg@vali.email\;\
+          "_dmarc.example.com": """v=DMARC1; p=reject;\
+ rua=mailto:dmarc.reports@valimail.com,mailto:dmarc_agg@vali.email;\
  ruf=mailto:dmarc.reports@valimail.com,mailto:dmarc_c0cb7153_afrf@vali.email""",
-          "_dmarc.example.org": """v=DMARC1\; p=none""",
-          "_dmarc.example.net": """v=DMARC1\; p=none\; sp=reject""",
+          "_dmarc.example.org": """v=DMARC1; p=none""",
+          "_dmarc.example.net": """v=DMARC1; p=none; sp=reject""",
           "_dmarc.sub.example.net": "",
-          "_dmarc.example.biz": """v=DMARC1\; p=none\; sp=quarantine\; np=reject""",
-          "_dmarc.nop.example.org": """v=DMARC1\; sp=quarantine\; np=reject""",
+          "_dmarc.example.biz": """v=DMARC1; p=none; sp=quarantine; np=reject""",
+          "_dmarc.nop.example.org": """v=DMARC1; sp=quarantine; np=reject""",
           "_dmarc.sub.example.biz": "",
           "_dmarc.sub2.example.biz": "",
           "sub.example.biz": None,
-          "sub2.example.biz": "host.example.biz"
+          "sub2.example.biz": "host.example.biz",
+          "_dmarc.bad.example.com": """v=DMARC1; p=none; reject""",
+          "_dmarc.gov": "v=DMARC1; p=reject; sp=none; np=reject; rua=mailto:dotgov_dmarc@cisa.dhs.gov; psd=y",
+          "_dmarc.example.gov": "",
+          "example.gov": "",
         }
         try:
             if isinstance(domain, bytes):
@@ -92,6 +99,18 @@ Y+vtSBczUiKERHv1yRbcaQtZFh5wtiRrN04BLUTD21MycBX5jYchHjPY/wIDAQAB""",
         res = authenticate_message(self.message2, "example.com", spf=False, dnsfunc=self.dnsfunc)
         self.assertEqual(res, "Authentication-Results: example.com; dkim=pass header.d=example.com header.i=@example.com; dmarc=pass (Used From Domain Record) header.from=example.com policy.dmarc=reject")
 
+    def test_authenticate_dmarc_bad(self):
+        res = authenticate_message(self.message10, "example.com", dkim=False, spf=False, dnsfunc=self.dnsfunc)
+        self.assertEqual(res, 'Authentication-Results: example.com; dmarc=permerror (missing tag or value: "v=DMARC1; p=none; reject") header.from=bad.example.com policy.dmarc=none')
+
+    def test_authenticate_dmarc_nofrom(self):
+        res = authenticate_message(self.message11, "example.com", dkim=False, spf=False, dnsfunc=self.dnsfunc)
+        self.assertEqual(res, 'Authentication-Results: example.com; dmarc=permerror (Unable to extract From domain: Test User) header.from=none policy.dmarc=none')
+
+    def test_authenticate_dmarc_psdsub(self):
+        res = authenticate_message(self.message12, "example.com", dkim=False, spf=False, dnsfunc=self.dnsfunc, dmarcbis=True)
+        self.assertEqual(res, 'Authentication-Results: example.com; dmarc=fail (Used Tree Walk, org one level below PSD) header.from=example.gov policy.dmarc=reject')
+
     def test_authenticate_dmarc_mult_from(self):
         self.maxDiff = None
         res = authenticate_message(self.message3, "example.com", prev='Authentication-Results: example.com; dkim=fail header.d=example.com header.i=@example.com; dkim=pass header.d=example.org header.i=@example.org', spf=False, dkim=False, dnsfunc=self.dnsfunc)
@@ -281,5 +300,10 @@ uEzxBDAr518Z8VFbR41in3W4Y3yCDgQlLlcETrS+zYcL
         res = [b"".join(x.split()) for x in res]
         self.assertEqual(res, headers)
 
+def _test():
+    return doctest.testmod(dmarc_lookup)
+
+
 if __name__ == '__main__':
+    _test()
     unittest.main()
